# -*- coding: utf-8 -*-

"""Common widgets and their bases."""

__all__ = ['IconButton']

from qtpy import QtWidgets


class IconButton(QtWidgets.QPushButton):
    """Common control for icon-only buttons."""
