#!/opt/concourse/worker/volumes/live/2bcbe701-7616-43e6-7186-dd9f074f408f/volume/anaconda-navigator_1632168928028/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/2bcbe701-7616-43e6-7186-dd9f074f408f/volume/anaconda-navigator_1632168928028/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])
